<?php
namespace Teknisa\CMP\Controllers;

use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Exception;
use Teknisa\CMP\Factories\Cmp99000GeracaoAfPrevisaoFactory;
use Teknisa\CMP\Services\Cmp99000GeracaoAfPrevisaoService;

class Cmp99000GeracaoAfPrevisaoController extends
    \Teknisa\GER\Controllers\TeknisaCrud
{
    protected $factory;
    protected $service;
    protected $crudService;
    protected $parameterBag;
    protected $environment;

    public function __construct(
        $crudService,
        $parameterBag,
        $environment,
        $factory,
        $service
    ) {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->environment = $environment;
        $this->factory = $factory;
        $this->service = $service;
    }

    public function getGeracaoAfPrevisao(Row $request, Response $response)
    {
        try {
            $params = $request->getParameters();

            if ($params["NRAPURPREC"] == null) {
                $params["NRAPURPREC"] = $this->service->generateCode(
                    "APURPRECO" . $params["CDFILCENTCMP"],
                    $params["NRORG"]
                );

                $rowsAffect = $this->factory->gerarProcedure($params);
             
                if($rowsAffect['P_ROWSAFFECT'] == "N"){
                   $response->setError(new Error("Não há dados para Geração de AF.", 500));
                   
                   return;
                }
            }
            
            $data = $this->factory->getItmaPurPrecData($params);
            
            $response->addDataSet(new DataSet('data', $data));

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function getSessionInfos(Row $request, Response $response)
    {
        $sessionInfos = [
            'CDFILIAL' => $this->environment->getCdFilial(),
            'CDOPERADOR' => $this->environment->getCdOperador()
        ];

        $response->addDataSet(new DataSet('data',  $sessionInfos));
    }
}
