<?php
/**
 * Created by PhpStorm.
 * User: MATHEUSSILVA
 * Date: 26/03/2015
 * Time: 11:16
 */

namespace Folha\Logic\ObrigacaoLegal\RAIS;

use \Folha\Entities\FpaFormrais;

use \Folha\Entities\GpePessoah;

use \Folha\Entities\GpeVinculoh;

use \Folha\Entities\GpeVinculom;

use \Geral\Entities\Estruturah;

use \HCM\Util\DateUtil;

class GeraRais
{
    /** @var  $rais Tipo00 */
    private $rais;
    /** @var  String $magnetico */
    private $magnetico;
    /** @var  $formRais FpaFormRais */
    private $formRais;

    private $ano;
    private $nrorg;
    private $dtmescompetenc;

    private $nrestruturasm;
    private $nrvinculosm;

    private $estrutResponsavel;
    private $pessoaResponsavel;
    private $retificacao;

    /**
     * Construtor
     * @param int $nrorg
     * @param String $dtmescompetenc
     */
    function __construct($nrorg, $dtmescompetenc)
    {
        $this->nrorg = $nrorg;
        $this->dtmescompetenc = $dtmescompetenc;
        $dtmescompetenc = DateUtil::getDataDeString($dtmescompetenc);
        $this->ano = $dtmescompetenc->format('Y');
    }

    public function setFormRais( $nrformrais)
    {
        $where = "NRFORMRAIS = :P_NRFORMRAIS AND NRORG = :P_NRORG";
        $params = array(
            "P_NRFORMRAIS" => $nrformrais,
            "P_NRORG" => $this->nrorg
        );
        $orderBy = "";
        $this->formRais = FpaFormrais::getObjectByWhere($where, $params, $orderBy);

        if(!$this->formRais){
            throw Exception::newException(Exception::ERRO_FORMULARIO_RAIS);
        }

        $this->retificacao = $this->formRais->getIdretifrais();
        $this->estrutResponsavel = $this->formRais->getNrestrutrespons();
        $this->pessoaResponsavel = $this->formRais->getNrpessoarespons();

        $this->aplicaFiltro($this->formRais->getDsordem(), $this->formRais->getNomeinicial(), $this->formRais->getNomefinal());
    }

    /**
     * Busca as estruturas e os vínculos com base na ordem selecionada
     * @param String $ordem
     * @param String|int $inicio
     * @param String|int $fim
     * @throws \Exception
     */
    private function aplicaFiltro( $ordem, $inicio, $fim)
    {
        try {
            switch ($ordem) {
                case('ALFABETICA'): {
                    if($inicio>$fim){
                        Exception::newException(Exception::ERRO_INICIO_MAIOR_FINAL);
                    }
                    $this->buscaAlfabetica( $inicio, $fim);
                    break;
                }
                case('MATRICULA'): {
                    if($inicio>$fim){
                        Exception::newException(Exception::ERRO_INICIO_MAIOR_FINAL);
                    }
                    $this->buscaMatricula( $inicio, $fim);
                    break;
                }
                case('ESTRUTURA'): {
                    $this->buscaEstrutura( $inicio, $fim);
                    break;
                }
                case('DATA_ADMISSAO'): {
                    if($inicio>$fim){
                        Exception::newException(Exception::ERRO_INICIO_MAIOR_FINAL);
                    }
                    $this->buscaDataAdmissao( $inicio, $fim);
                    break;
                }
                default: {
                    throw new \Exception("Opção de filtro não encontrada");
                    break;
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Busca as estrutura e os vínculos que entrarão na RAIS pelo nome dos vícnulos
     * @param string $inicio
     * @param string $fim
     * @throws \Exception
     */
    private function buscaAlfabetica( $inicio, $fim)
    {
        try {
            $where = "NMVINCULOM BETWEEN :P_INICIO AND :P_FINAL AND NRORG = :P_NRORG AND DTADMISSAOVINC <= :P_DTADMISSAOVINC AND (DTRESCISAOVINC >= :P_DTRESCISAOVINC OR DTRESCISAOVINC IS NULL)";
            $params = array(
                'P_INICIO' => $inicio,
                'P_FINAL' => $fim,
                'P_NRORG' => $this->nrorg,
                'P_DTADMISSAOVINC' => "31/12/{$this->ano}",
                'P_DTRESCISAOVINC' => "01/01/{$this->ano}"
            );
            $orderBy = "";
            $vinculosm = GpeVinculom::getObjectsByWhere($where, $params, $orderBy); /** @var GpeVinculom[] $vinculosm */

            $nrestruturas = array();

            foreach($vinculosm as $vinculom){
                $this->nrvinculosm[] = $vinculom->getNrvinculom();
                $where = "NRVINCULOM = :P_NRVINCULOM AND NRORG = :P_NRORG AND DTMESCOMPETENC <= :P_DTMESCOMPETENC";
                $params = array(
                    'P_DTMESCOMPETENC' => $this->dtmescompetenc,
                    'P_NRVINCULOM' => $vinculom->getNrvinculom(),
                    'P_NRORG' => $this->nrorg
                );
                $orderBy = "DTMESCOMPETENC DESC";
                $vinculoh = GpeVinculoh::getObjectByWhere($where, $params, $orderBy); /** @var GpeVinculoh $vinculoh */
                $nrestruturas[] = $vinculoh->getNrestrutlegal();
            }
            $nrestruturas = array_filter($nrestruturas);
            $this->nrestruturasm = array_unique($nrestruturas);
            $this->nrvinculosm = array_filter($this->nrvinculosm);

        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * Busca as estrutura e os vínculos que entrarão na RAIS pela matricula
     * @param int $inicio
     * @param int $fim
     * @throws \Exception
     */
    private function buscaMatricula( $inicio, $fim)
    {
        try {
            $where = "CDMATRICULA BETWEEN :P_INICIO AND :P_FINAL AND NRORG = :P_NRORG AND  DTADMISSAOVINC <= :P_DTADMISSAOVINC AND (DTRESCISAOVINC >= :P_DTRESCISAOVINC OR DTRESCISAOVINC IS NULL)";
            $params = array(
                'P_INICIO' => $inicio,
                'P_FINAL' => $fim,
                'P_NRORG' => $this->nrorg,
                'P_DTADMISSAOVINC' => "31/12/{$this->ano}",
                'P_DTRESCISAOVINC' => "01/01/{$this->ano}"
            );
            $orderBy = "";
            $vinculosm = GpeVinculom::getObjectsByWhere($where, $params, $orderBy);
            /** @var GpeVinculom[] $vinculosm */

            $nrestruturas = array();

            foreach ($vinculosm as $vinculom) {
                $this->nrvinculosm[] = $vinculom->getNrvinculom();
                $where = "NRVINCULOM = :P_NRVINCULOM AND NRORG = :P_NRORG AND DTMESCOMPETENC <= :P_DTMESCOMPETENC";
                $params = array(
                    'P_DTMESCOMPETENC' => $this->dtmescompetenc,
                    'P_NRVINCULOM' => $vinculom->getNrvinculom(),
                    'P_NRORG' => $this->nrorg
                );
                $orderBy = "DTMESCOMPETENC DESC";
                $vinculoh = GpeVinculoh::getObjectByWhere($where, $params, $orderBy);
                /** @var GpeVinculoh $vinculoh */
                $nrestruturas[] = $vinculoh->getNrestrutlegal();
            }
            $this->nrestruturasm[] = array_unique($nrestruturas);
        }catch (\Exception $e){
            throw $e;
        }

    }

    /**
     * Busca as estrutura e os vínculos que entrarão na RAIS pela Estrutura
     * @param int $estrutlegal
     * @param int $estrutger
     * @throws \Exception
     */
    private function buscaEstrutura( $estrutlegal, $estrutger)
    {
        $where = "NRORG = :P_NRORG AND DTMESCOMPETENC <= :P_DTMESCOMPETENC AND NRESTRUTLEGAL = :P_NRESTRUTLEGAL AND NRESTRUTGEREN = :P_NRESTRUTGEREN";
        $params = array(
            'P_DTMESCOMPETENC' => $this->dtmescompetenc,
            'P_NRORG' => $this->nrorg,
            'P_NRESTRUTLEGAL' => $estrutlegal,
            'P_NRESTRUTGEREN' => $estrutger

        );
        $orderBy = "DTMESCOMPETENC DESC";
        $vinculosh = GpeVinculoh::getObjectsByWhere($where, $params, $orderBy); /** @var GpeVinculoh[] $vinculosh */

        $nrvinculosm = array();
        $nrestruturasm = array();

        foreach($vinculosh as $vinculoh){
            $nrvinculosm[] = $vinculoh->getNrvinculom();
            $nrestruturasm[] = $vinculoh->getNrestrutlegal();
        }

        $this->nrvinculosm = array_unique($nrvinculosm);
        $this->nrestruturasm = array_unique($nrestruturasm);

    }

    /**
     * Busca as estrutura e os vínculos que entrarão na RAIS pela data de admissão dos vícnulos
     * @param string $inicio
     * @param string $fim
     * @throws \Exception
     */
    private function buscaDataAdmissao( $inicio, $fim)
    {
        try {
            $where = "DTADMISSAOVINC BETWEEN :P_INICIO AND :P_FINAL AND NRORG = :P_NRORG AND DTADMISSAOVINC <= :P_DTADMISSAOVINC AND (DTRESCISAOVINC >= :P_DTRESCISAOVINC OR DTRESCISAOVINC IS NULL)";
            $params = array(
                'P_INICIO' => $inicio,
                'P_FINAL' => $fim,
                'P_NRORG' => $this->nrorg,
                'P_DTADMISSAOVINC' => "31/12/{$this->ano}",
                'P_DTRESCISAOVINC' => "01/01/{$this->ano}"
            );
            $orderBy = "";
            $vinculosm = GpeVinculom::getObjectsByWhere($where, $params, $orderBy); /** @var GpeVinculom[] $vinculosm */

            $nrestruturas = array();

            foreach($vinculosm as $vinculom){
                $this->nrvinculosm[] = $vinculom->getNrvinculom();
                $where = "NRVINCULOM = :P_NRVINCULOM AND NRORG = :P_NRORG AND DTMESCOMPETENC <= :P_DTMESCOMPETENC";
                $params = array(
                    'P_DTMESCOMPETENC' => $this->dtmescompetenc,
                    'P_NRVINCULOM' => $vinculom->getNrvinculom(),
                    'P_NRORG' => $this->nrorg
                );
                $orderBy = "DTMESCOMPETENC DESC";
                $vinculoh = GpeVinculoh::getObjectByWhere($where, $params, $orderBy); /** @var GpeVinculoh $vinculoh */
                $nrestruturas[] = $vinculoh->getNrestrutlegal();
            }
            $this->nrestruturasm[] = array_unique($nrestruturas);

        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function buscaResponsavel()
    {
        //BUSCA A ESTRUTURA RESPONSÁVEL
        $where = "NRESTRUTURAM = :P_NRESTRUTURAM AND NRORG = :P_NRORG AND DTMESCOMPETENC <= :P_DTMESCOMPETENC";
        $params = array(
            'P_NRESTRUTURAM' => $this->estrutResponsavel,
            'P_NRORG' => $this->nrorg,
            'P_DTMESCOMPETENC' => $this->dtmescompetenc
        );
        $orderBy = "DTMESCOMPETENC DESC"; /** @var Estruturah $estruturah */
        $estruturah = Estruturah::getObjectByWhere($where, $params, $orderBy);  /** @var Estruturah $estruturah */

        if(!$estruturah){
            throw Exception::newException(Exception::ERRO_HIST_ESTRUT_RESP);
        }
        $this->estrutResponsavel = $estruturah;

        //BUSCA O VÍNCULO
        $where = "NRPESSOA = :P_NRPESSOA AND NRORG = :P_NRORG AND DTMESCOMPETENC <= :P_DTMESCOMPETENC";
        $params = array(
            'P_NRPESSOA' => $this->pessoaResponsavel,
            'P_NRORG' => $this->nrorg,
            'P_DTMESCOMPETENC' => $this->dtmescompetenc
        );
        $orderBy = "DTMESCOMPETENC DESC";
        $pessoah = GpePessoah::getObjectByWhere($where, $params, $orderBy); /** @var GpePessoah $pessoah */

        if(!$pessoah){
            throw Exception::newException(Exception::ERRO_HIST_PESSOA_RESP);
        }

        $this->pessoaResponsavel = $pessoah;
    }

    public function gerar( $grpInciden, $centralPgto = null, $sindicato = null)
    {
        $this->buscaResponsavel();
        $this->rais = new Tipo00($this->estrutResponsavel, $this->pessoaResponsavel, $this->nrorg, $this->ano, $this->retificacao, $this->formRais->getNrformrais());
        $this->rais->setTipo01(implode(', ', $this->nrestruturasm), $centralPgto, $sindicato);
        $this->rais->setTipo02($grpInciden, "AND (NRVINCULOM = ".implode(' OR NRVINCULOM = ', $this->nrvinculosm).")");
        $this->rais->setTipo09();
    }

    public function preparaDados()
    {
        $info = $this->rais->getMagnetico();
        $this->magnetico = '';
        for($c = 0; $c<count($info);$c++){
            $this->magnetico .= str_pad($c+1, 6, '0', STR_PAD_LEFT).$info[$c]."\n";
        }
    }

    public function download()
    {
        $data = DateUtil::getDataAtual();
        $nome = "RAIS_".$this->rais->getRazaoSocial().'_'.$data->format('dmYhis').".dec";
        $path = "..\\files\\RAIS";

        if (!file_exists($path)) {
            mkdir($path, 0777, true);
        }

        $path = realpath($path);
        $caminho = $path.DIRECTORY_SEPARATOR.$nome;
        file_put_contents($caminho, $this->magnetico);

        $arquivo = new \TkFile(realpath($caminho)); //variável para o download do arquivo
        $arquivo->download();
    }

} 