<?php
/**
 * Registro5CNABSantander
 *
 * @package    Logic
 * @subpackage Registro/Cnab
 *
 * @author     thalesvalias - 30/07/2012
 *
 */
class Registro5CNABSantander extends RegistroCNAB{

    const REGISTRO_5 = '5';


    /**
     * Gera Tipo de Registro 5 - Registro Trailer de Lote.
     *
     * @return void
     *
     * @author thalesvalias - 30/07/2012
     */
    public function geraRegistro5(){
        $this->_geraRegistro();
    }


    /**
     * Atribui os dados no atributo da classe Pai.
     *
     * @return void
     *
     * @author thalesvalias - 30/07/2012
     */
    private function _geraRegistro(){
        // Bloco 1 ao 3 eh gerado no pai.
        parent::geraRegistro(self::REGISTRO_5, CNABSantander::CODIGO_BANCO_SANTANDER, self::LOTE_SERVICO);

        // Bloco 4 | 009-017 | Filler (Caracter branco).
        $this->atribuiDado(parent::CARACTER_BRANCO, 9, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 5 | 018-023 | Quantidade de Registros do Lote (Registros Tipo 1, 3, 5).
        // Calculado apos a geracao de todos os registros do arquivo (a linha eh modificada no PercorreCaged).
        $this->atribuiDado(parent::CARACTER_ZERO, 6, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 6 | 024-041 | Somatoria dos Valores.
        // Valor obtido pela somatoria dos valores de Creditos / Pagamentos dos registros de detalhe (Registros do Tipo 3 / Segmento A).
        $this->atribuiDado(parent::CARACTER_ZERO, 18, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 7 | 042-059 | Somatoria Quantidade Moeda (Registro Tipo 3). Mesmo valor do bloco anterior (6).
        $this->atribuiDado(parent::CARACTER_ZERO, 18, parent::CARACTER_ZERO, parent::PREENCHE_DIREITA);

        // Bloco 8 | 060-065 | Numero Aviso de Debito.
        // Numero atribuido pelo Banco para identificar um debito efetuado na Conta Corrente a partir
        // do(s) pagamento(s) efetivado(s), visando facilitar a Conciliacao Bancaria.
        $this->atribuiDado(parent::CARACTER_ZERO, 6, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 9 | 066-230 | Filler.
        $this->atribuiDado(parent::CARACTER_BRANCO, 165, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 25 | 231-240 | Ocorrencias para o Retorno.
        // Codigo adotado para identificar as ocorrencias detectadas no processamento.
        // Pode-se informar ate 5 ocorrencias simultaneamente, cada uma delas codificada com dois
        // digitos, conforme relacao das paginas 32 e 33.
        $this->atribuiDado(parent::CARACTER_BRANCO, 10, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
    }


}